<?php
/**
 * @file
 * Administration pages for the 'jQuery UI filter' module.
 */

////////////////////////////////////////////////////////////////////////////////
// Admin settings form
////////////////////////////////////////////////////////////////////////////////

/**
 * Form builder; Settings page for the 'jQuery UI filter' module.
 *
 * @ingroup forms
 */
function jquery_ui_filter_admin_settings() {
  drupal_add_js(drupal_get_path('module', 'jquery_ui_filter') .'/jquery_ui_filter.admin.js');

  // General
  $form['general'] = array(
    '#type' => 'fieldset',
    '#title' => 'General settings',
  );
  $form['general']['jquery_ui_filter_header_tag'] = array(
    '#type' => 'select',
    '#title' => t('Header tag'),
    '#description' => t('The selected header tag is used to generate the jQuery UI Accordion and Tabs when the jQuery UI filter is assigned to an input format.'),
    '#options' => array(
      'h1' => 'h1',
      'h2' => 'h2',
      'h3' => 'h3',
      'h4' => 'h4',
      'h5' => 'h5',
      'h6' => 'h6',
    ),
    '#required' => 1,
    '#default_value' => variable_get('jquery_ui_filter_header_tag', 'h3')
  );

  $form['general']['jquery_ui_filter_disabled_pages'] = array(
    '#type' => 'textarea',
    '#title' => t('Disable jQuery UI widgets on specific pages'),
    '#description' => t("Enter one page per line as Drupal paths. The '*' character is a wildcard. Example paths are '@example'", array('@example' => 'print*')),
    '#default_value' => variable_get('jquery_ui_filter_disabled_pages', 'print*'),
  );

  $form['general']['jquery_ui_filter_css'] = array(
    '#type' => 'textfield',
    '#title' => t('jQuery UI theme CSS'),
    '#description' => t('The jQuery UI module does not automatically include the required CSS. Please use the default path, enter a path for your custom rolled jQuery UI CSS, or leaving this blank to allow your theme to include the required files.'),
    '#default_value' => variable_get('jquery_ui_filter_css', jquery_ui_get_path() .'/themes/base/jquery-ui.css'),
  );

  /* Accordion */
  /* http://www.phpeveryday.com/articles/jQuery-UI-Configuration-Accordion-Widget-P997.html */

  $accordion_options = variable_get('jquery_ui_filter_accordion_options', array());

  // Accordion
  $form['jquery_ui_filter_accordion_options'] = array(
    '#type' => 'fieldset',
    '#title' => 'Accordion options',
    '#description' => t('Learn more about <a href="@href">jQuery UI accordion options</a>.', array('@href' => 'http://jqueryui.com/demos/accordion/')),
    '#tree' => TRUE,
  );

  // Accordion: active
  $form['jquery_ui_filter_accordion_options']['active'] = array(
    '#title' => t('active'),
    '#type' => 'select',
    '#default_value' => (isset($accordion_options['active'])) ? $accordion_options['active'] : 1,
    '#options' => array('false' => 'none', 0 => 'first child'),
    '#description' => t('Selector for the active element. Needs collapsible: true..'),
  );

  // Accordion: animated
  $form['jquery_ui_filter_accordion_options']['animated'] = array(
    '#title' => t('animated'),
    '#type' => 'select',
    '#default_value' => (isset($accordion_options['animated'])) ? $accordion_options['animated'] : 'slide',
    '#options' => array(
      0 => 'false',
      'slide' => 'slide',
      'bounceslide' => 'bounceslide',
    ),
    '#description' => t("Choose your favorite animation, or disable them (set to false)."),
  );

  // Accordion: auto Height
  $form['jquery_ui_filter_accordion_options']['autoHeight'] = array(
    '#title' => t('autoHeight'),
    '#type' => 'select',
    '#default_value' => (isset($accordion_options['autoHeight'])) ? $accordion_options['autoHeight'] : TRUE,
    '#options' => array('false' => 'false', 'true' => 'true'),
    '#description' => t('If set, the highest content part is used as height reference for all other parts. Provides more consistent animations.'),
  );

  // Accordion: collapsible
  $form['jquery_ui_filter_accordion_options']['collapsible'] = array(
    '#title' => t('collapsible'),
    '#type' => 'select',
    '#default_value' => (empty($accordion_options['collapsible'])) ? FALSE : TRUE,
    '#options' => array('false' => 'false', 'true' => 'true'),
    '#description' => t('Whether all the sections can be closed at once. Allows collapsing the active section by the triggering event (click is the default).'),
  );

  // Accordion: event
  $form['jquery_ui_filter_accordion_options']['event'] = array(
    '#title' => t('event'),
    '#type' => 'select',
    '#default_value' => (isset($accordion_options['event'])) ? $accordion_options['event'] : 'click',
    '#options' => array('click' => 'click', 'mouseover' => 'mouseover'),
    '#description' => t('The event on which to trigger the accordion.'),
  );

  // Accordion: fillSpace [SKIPPED]
  // Since accordion is embedded inside content there is no way of knowing what
  // the container's height will be which makes this settings unpredicable.

  // Accordion: navigation [SKIPPED]
  // Deprecate navigation options - http://bugs.jqueryui.com/ticket/5869

  // Accordion: Global
  $form['jquery_ui_filter_accordion_options']['jquery_ui_filter_accordion_options_global'] = array(
    '#title' => t('Globally apply all the above accordion options to all instances of the jQuery UI accordion widget.'),
    '#type' => 'checkbox',
    '#default_value' => variable_get('jquery_ui_filter_accordion_options_global', FALSE),
    '#description' => t("By default the above options are only applied to accordion widgets generated by the jQuery UI filter. If checked, the selected options below will be applied as the defaults for all jQuery UI accordion widgets."),
    '#tree' => FALSE,
  );

  /* Tabs */

  $tabs_options = variable_get('jquery_ui_filter_tabs_options', array());

  // Tabs
  $form['jquery_ui_filter_tabs_options'] = array(
    '#type' => 'fieldset',
    '#title' => 'Tabs options',
    '#description' => t('Learn more about <a href="@href">jQuery UI tabs options</a>.', array('@href' => 'http://jqueryui.com/demos/tabs/')),
    '#tree' => TRUE,
  );

  // Tabs: collapsible
  $form['jquery_ui_filter_tabs_options']['collapsible'] = array(
    '#title' => t('collapsible'),
    '#type' => 'select',
    '#default_value' => (empty($tabs_options['collapsible'])) ? FALSE : TRUE,
    '#options' => array('false' => 'false', 'true' => 'true'),
    '#description' => t("Set to 'true' to allow an already selected tab to become unselected again upon reselection."),
  );

  // Tabs: cookie (require jquery plugin)
  if (module_exists('jquery_plugin')) {
    $form['jquery_ui_filter_tabs_options']['cookie'] = array(
      '#title' => t('cookie'),
      '#type' => 'checkbox',
      '#default_value' => (empty($tabs_options['cookie'])) ? FALSE : TRUE,
      '#description' => t("Store the latest selected tab in a cookie. The cookie is then used to determine the initially selected tab if the selected option is not defined. (Requires cookie plugin)"),
    );
    $form['jquery_ui_filter_tabs_options']['cookie_options'] = array(
      '#type' => 'fieldset',
      '#title' => t('cookie options'),
      '#attributes' => array('id' => 'jquery-ui-filter-cookie-options'),
    );
    $form['jquery_ui_filter_tabs_options']['cookie_options']['expires'] = array(
      '#title' => t('expires'),
      '#type' => 'textfield',
      '#field_suffix' => t('days'),
      '#default_value' => (isset($tabs_options['cookie']['expires'])) ? $tabs_options['cookie']['expires'] : 7,
      '#size' => 3,
      '#maxlength' => 3,
      '#description' => t('Cookie lifetime in days. If omitted, the cookie is a session cookie.'),
    );
    $form['jquery_ui_filter_tabs_options']['cookie_options']['secure'] = array(
      '#title' => t('secure'),
      '#type' => 'select',
      '#default_value' => (empty($tabs_options['cookie']['secure'])) ? FALSE : TRUE,
      '#options' => array('false' => 'false', 'true' => 'true'),
      '#description' => t('If true, the cookie transmission requires a secure protocol (https).'),
    );
  }

  // Tabs: event
  $form['jquery_ui_filter_tabs_options']['event'] = array(
    '#title' => t('event'),
    '#type' => 'select',
    '#default_value' => (isset($tabs_options['event'])) ? $tabs_options['event'] : 'click',
    '#options' => array('click' => 'click', 'mouseover' => 'mouseover'),
    '#description' => t('The type of event to be used for selecting a tab.'),
  );

  // Tabs: fx
  $form['jquery_ui_filter_tabs_options']['fx'] = array(
    '#title' => t('fx'),
    '#type' => 'checkbox',
    '#default_value' => (empty($tabs_options['fx'])) ? FALSE : TRUE,
    '#description' => t('Enable animations for hiding and showing tab panels.'),
  );
  $form['jquery_ui_filter_tabs_options']['fx_options'] = array(
    '#title' => t('fx options'),
    '#type' => 'fieldset',
    '#attributes' => array('id' => 'jquery-ui-filter-fx-options'),
  );
  $form['jquery_ui_filter_tabs_options']['fx_options'][0] = array(
    '#type' => 'fieldset',
    '#title' => t('Hide'),
  ) + _jquery_ui_filter_admin_settings_fx_options( isset($tabs_options['fx'][0]) ? $tabs_options['fx'][0] : array() );
  $form['jquery_ui_filter_tabs_options']['fx_options'][1] = array(
    '#type' => 'fieldset',
    '#title' => t('Show'),
  ) + _jquery_ui_filter_admin_settings_fx_options( isset($tabs_options['fx'][1]) ? $tabs_options['fx'][0] : array() );

  // Tabs: paging
  $form['jquery_ui_filter_tabs_options']['paging'] = array(
    '#title' => t('paging'),
    '#type' => 'checkbox',
    '#default_value' => (isset($tabs_options['paging'])) ? $tabs_options['paging'] : TRUE,
    '#description' => t("Add 'back' and 'next' button to page through tabs."),
  );
  $form['jquery_ui_filter_tabs_options']['paging_options'] = array(
    '#title' => t('paging options'),
    '#type' => 'fieldset',
    '#attributes' => array('id' => 'jquery-ui-filter-paging-options'),
  );
  $form['jquery_ui_filter_tabs_options']['paging_options']['back'] = array(
    '#title' => t('back label'),
    '#type' => 'textfield',
    '#default_value' => (isset($tabs_options['paging']['back'])) ? $tabs_options['paging']['back'] : '&laquo; Previous',
  );
  $form['jquery_ui_filter_tabs_options']['paging_options']['next'] = array(
    '#title' => t('next label'),
    '#type' => 'textfield',
    '#default_value' => (isset($tabs_options['paging']['next'])) ? $tabs_options['paging']['next'] : 'Next &raquo;',
  );

  // Tabs: selected
  $form['jquery_ui_filter_tabs_options']['selected'] = array(
    '#title' => t('selected'),
    '#type' => 'select',
    '#default_value' => (isset($tabs_options['selected'])) ? $tabs_options['selected'] : '',
    '#options' => array('' => '', '-1' => '-1'),
    '#description' => t("To set all tabs to unselected pass -1 as value."),
  );

  // Tabs: Global
  $form['jquery_ui_filter_tabs_options']['jquery_ui_filter_tabs_options_global'] = array(
    '#title' => t('Globally apply all the above tabs options to all instances of the jQuery UI tabs widget.'),
    '#type' => 'checkbox',
    '#default_value' => variable_get('jquery_ui_filter_tabs_options_global', FALSE),
    '#description' => t("By default the above options are only applied to tabs widgets generated by the jQuery UI filter. If checked, the selected options below will be applied as the defaults for all jQuery UI tabs widgets."),
    '#tree' => FALSE,
  );

  // Submit
  $form['buttons']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save configuration'),
  );
  $form['buttons']['reset'] = array(
    '#type' => 'submit',
    '#value' => t('Reset to defaults'),
  );

  return $form;
}

/**
 * Submit handler; Settings page for the 'jQuery UI filter' module.
 *
 * @ingroup forms
 */
function jquery_ui_filter_admin_settings_submit($form, &$form_state) {
  $op = isset($form_state['values']['op']) ? $form_state['values']['op'] : '';
  $values = $form_state['values'];
  if ($op == t('Reset to defaults')) {
    variable_del('jquery_ui_filter_header_tag');
    variable_del('jquery_ui_filter_disabled_pages');
    variable_del('jquery_ui_filter_css');
    variable_del('jquery_ui_filter_tabs_options');
    variable_del('jquery_ui_filter_tabs_options_global');
    variable_del('jquery_ui_filter_accordion_options');
    variable_del('jquery_ui_filter_accordion_options_global');

    drupal_set_message(t('The configuration options have been reset to their default values.'));
  }
  else {
    variable_set('jquery_ui_filter_header_tag', $values['jquery_ui_filter_header_tag']);
    variable_set('jquery_ui_filter_disabled_pages', $values['jquery_ui_filter_disabled_pages']);
    variable_set('jquery_ui_filter_css', $values['jquery_ui_filter_css']);
    variable_set('jquery_ui_filter_tabs_options', _jquery_ui_filter_set_options($values['jquery_ui_filter_tabs_options']));
    variable_set('jquery_ui_filter_tabs_options_global', $values['jquery_ui_filter_tabs_options_global']);
    variable_set('jquery_ui_filter_accordion_options', _jquery_ui_filter_set_options($values['jquery_ui_filter_accordion_options']));
    variable_set('jquery_ui_filter_accordion_options_global', $values['jquery_ui_filter_accordion_options_global']);
    drupal_set_message(t('The configuration options have been saved.'));
  }

  // DEBUG:
  // dpm(variable_get('jquery_ui_filter_tabs_options', array()));
  // dpm(variable_get('jquery_ui_filter_accordion_options', array()));
}


////////////////////////////////////////////////////////////////////////////////
// Demo settings form
////////////////////////////////////////////////////////////////////////////////

function jquery_ui_filter_demo_settings($form_state) {
  if (isset($form_state['post']['op'])) {
    if ($form_state['post']['op'] == t('Reset')) {
      drupal_goto($_GET['q']);
    }
    $tabs_source = $form_state['post']['tabs_source'];
    $accordion_source = $form_state['post']['accordion_source'];
  }
  else {
    $header_tag = variable_get('jquery_ui_filter_header_tag', 'h3');
    $section_html = '<p>Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas. Vestibulum tortor quam, feugiat vitae, ultricies eget, tempor sit amet, ante. Donec eu libero sit amet quam egestas semper. Aenean ultricies mi vitae est. Mauris placerat eleifend leo.</p>
<ul>
<li>Lorem ipsum dolor sit amet, consectetuer adipiscing elit.</li>
<li>Aliquam tincidunt mauris eu risus.</li>
<li>Vestibulum auctor dapibus neque.</li>
</ul>';
    $filtered_html = "<$header_tag>Section I</$header_tag>\n$section_html\n<$header_tag>Section II</$header_tag>\n$section_html\n<$header_tag>Section III</$header_tag>\n$section_html";
    $tabs_source = "<p>[tabs]</p>\n$filtered_html<p>[/tabs]</p>\n";
    $accordion_source = "<p>[accordion]</p>\n$filtered_html<p>[/accordion]</p>\n";
  }

  $form = array('#redirect' => FALSE);

  // Demo
  $form['demo'] = array(
    '#value' => '<h2>'. t('Tabs') .'</h2>'. _jquery_ui_filter_process_callback($tabs_source) .'<br/>'.
      '<h2>'. t('Accordion') .'</h2>'. _jquery_ui_filter_process_callback($accordion_source) .'<br/>',
  );

  // HTML
  $form['html'] = array(
    '#type' => 'fieldset',
    '#title' => t('HTML source'),
    '#description' => jquery_ui_filter_filter_tips(NULL, NULL),
  );
  $form['html']['tabs_source'] = array(
    '#type' => 'textarea',
    '#title' => t('Tabs source'),
    '#default_value' => $tabs_source,
    '#required' => TRUE,
  );
  $form['html']['accordion_source'] = array(
    '#type' => 'textarea',
    '#title' => t('Accordion source'),
    '#default_value' => $accordion_source,
    '#required' => TRUE,
  );
  $form['html']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Update'),
  );
  $form['html']['reset'] = array(
    '#type' => 'submit',
    '#value' => t('Reset'),
  );

  return $form;
}


////////////////////////////////////////////////////////////////////////////////
// jQuery UI options functions
////////////////////////////////////////////////////////////////////////////////

/**
 * Get Fx hide or show options.
 */
function _jquery_ui_filter_admin_settings_fx_options($default_values = array()) {
  $fx_options = array();
  $fx_options['opacity'] = array(
    '#title' => t('opacity'),
    '#type' => 'checkbox',
    // Usually the best tab effect is just to use opacity (aka fade in/out)
    '#default_value' => (isset($default_values['opacity'])) ? $default_values['opacity'] : TRUE,
    '#return_value' => 'toggle'
  );
  $fx_options['height'] = array(
    '#title' => t('height'),
    '#type' => 'checkbox',
    '#default_value' => (empty($default_values['height'])) ? FALSE : 'toggle',
    '#return_value' => 'toggle'
  );
  // width: [SKIPPED]
  // Adjusting the width cause unpredictable results so don't include it.
  $fx_options['duration'] = array(
    '#title' => t('duration'),
    '#type' => 'select',
    '#default_value' => (isset($default_values['duration'])) ? $default_values['duration'] : 'normal',
    '#options' => array('normal' => 'normal', 'slow' => 'slow', 'medium' => 'medium', 'fast' => 'fast'),
  );
  return $fx_options;
}

/**
 * Set jQuery UI tabs or accordion options objects.
 */
function _jquery_ui_filter_set_options($options) {
  _jquery_ui_filter_set_options_types_recursive($options);

  // Clean up fx options by removing toggle options that are set to 0.
  if (isset($options['fx_options'])) {
    $options['fx_options'][0] = array_filter($options['fx_options'][0]);
    $options['fx_options'][1] = array_filter($options['fx_options'][1]);
  }

  // Set options object for enabled options (ie fx, cookie, etc...);
  foreach ($options as $name => $value) {
    if (isset($options[$name .'_options'])) {
      if ($value) {
        $options[$name] = $options[$name .'_options'];
      }
      unset($options[$name .'_options']);
    }
  }

  return $options;
}

/**
 * Convert jQuery UI tabs or accordion options numeric strings to integers.
 */
function _jquery_ui_filter_set_options_types_recursive(&$options) {
  foreach ($options as $name => $value) {
    if (is_numeric($value)) {
      $options[$name] = (int) $value;
    }
    elseif (is_array($value)) {
      _jquery_ui_filter_set_options_types_recursive($options[$name]);
    }
  }
}
